library(nnet)
#если пакет ещё небыл ранее установлен - нужно один раз выполнить код
#install.packages("nnet",dependencies = TRUE)
#и перезапустить R

setwd("C:/R/nnet_test/") #папка из которой будут читаться csv файлы, и в которую сохраняться найденные веса


#разные параметры для обучения
NNET_INPUT_SIZE  <- 100 #размер входного слоя нейронки (число бар которые подаются в нейронку для обучения или прогноза)
NNET_HIDDEN_SIZE <- 2   #размер скрытого слоя нейронки
DATE_FROM <- "2017.01.01" #дата с которой начинается обучение
DATE_TO   <- "2017.01.31" #дата до которой пойдёт обучение
FILE_NAME <- c("EURUSD_M30.csv", "GBPUSD_M30.csv")
# FILE_NAMES <- c("EURUSD_M30.csv") #пример если файл всего один
#вообще, в этом примере если обучать одну нейронку торговать два символа - то они должны иметь цены в примерно одинаковом интервале.
#Например обучать на золото и евро одновременно - нельзя. А фунт и евро вроде сошлись.


#создание таблицы для тренировки нейронки.
#тренировать нейронку на чистых ценах нельзя, тут с помощью функции diff() создаётся вектор прироста цен {..., open[2]-open[3], open[1]-open[2], open[0]-open[1]}
trainTableTargetCol <- NNET_INPUT_SIZE+1
trainTable <- matrix(NA, nrow=0, ncol=trainTableTargetCol)
for(filenameIt in 1:length(FILE_NAMES)){
  csvFile <- read.csv(FILE_NAMES[filenameIt], sep = "\t")
  if(ncol(csvFile)==8){
    colnames(csvFile) <- c("Date", "Open", "High", "Low", "Close", "Tick volume", "Trade volume", "Spread")
    csvFile[,"Date"] <- as.character(csvFile[,"Date"])
  }else{
    colnames(csvFile) <- c("Date", "Time", "Open", "High", "Low", "Close", "Tick volume", "Trade volume", "Spread")
    csvFile[,"Date"] <- as.character(csvFile[,"Date"])
    csvFile[,"Time"] <- as.character(csvFile[,"Time"])
  }
  trainIndexes <- which(csvFile[,"Date"] >= DATE_FROM & csvFile[,"Date"] <= DATE_TO)
  if(trainIndexes[1]-1 < NNET_INPUT_SIZE+1){
    stop("Ошибка. В csv недостаточно строк для обучения")
  }
  trainTableAppend <- matrix(NA, nrow=length(trainIndexes), ncol=trainTableTargetCol)
  for(i in 1:length(trainIndexes)){
    trainTableAppend[i,] <- diff(csvFile[(trainIndexes[i]-NNET_INPUT_SIZE-1):trainIndexes[i],"Open"])
  }
  trainTable <- rbind(trainTable, trainTableAppend)
}


#обучение нейронки
nnetResult <- nnet(x = trainTable[,1:NNET_INPUT_SIZE],
                   y = trainTable[,trainTableTargetCol],
                   size = NNET_HIDDEN_SIZE,
                   linout = TRUE,
                   maxit = 1000000,
                   trace = TRUE,
                   MaxNWts = 1000000)

#сохраним веса нейронки в файл
cat(nnetResult$wts, sep=",", file="weights.txt")
#дальше эти веса нужно вставить в советник в массив nnet_weights[]

#в mql советнике нужно в коде поменять ещё два числа - 
# static const int nnet_bars = значение NNET_INPUT_SIZE из R
# static const int nnet_hidden = значение NNET_HIDDEN_SIZE из R

